!> \file bmelt-ant-regions-initmip_mod.f90
!! Module qui calcule la fusion basale (grounded ou ice shelves).
!<

!> \namespace BMELT_ANT_REGIONS_INITMIP
!! Module qui calcule la fusion basale (grounded ou ice shelves)
!! \author CatRitz, aquiquet
!! \date juillet
!! @note pour les ice shelves Antarctique, tient compte des diffrentes rgions
!! A choisir dans le module_choix
!! @note Used module
!! @note   - use module3D_phy
!<
MODULE  BMELT_ANT_REGIONS_LARMIP         ! cat juillet 2005 - afq 2018

  use module3D_phy
  use netcdf
  use io_netcdf_grisli
  
  implicit none

  REAL,dimension(nx,ny) ::  bmgrz       !< fusion basale a la grounding zone
  real,dimension(nx,ny) ::  bmshelf     !< fusion basale sous shelf

  REAL,dimension(nx,ny) :: typeshelf    !< Type de shelf Ronne->1 Ross ->2 ....

  integer, parameter :: nb_reg = 18

  real,dimension (nb_reg) :: bmelt_regions, bmgrz_regions
  real :: bmelt_talus,bmgrz_talus       !< Au dela du talus continental
  real :: bmelt_coef                    !< Coef pour corriger les valeurs std de bmelt (lu dans namelist)
  real, parameter :: depth_talus=-2500. !< Profondeur du talus continental

  ! For initMIP, we need to read an anomaly of bmelt...:
  integer :: flag_dist
  real,dimension(nx,ny) :: coef_bmelt_dist
  
CONTAINS
  !-------------------------------------------------------------------------------
  subroutine init_bmelt


    ! Cette routine fait l'initialisation pour la fusion basale.
    ! Elle est appele par inputfile-vec-0.5.f90

    real*8, dimension(:,:),   pointer    :: tab               !< tableau 2d real pointer
    real :: bmelt_dist0
    character(len=100) :: file_number_shelves ! fichier avec les zones ice-shelves
    character(len=100) :: file_reg_larmip
    real,dimension(nx,ny) :: dist_talus  ! distance au talus
    character(len=100) :: file_dist_talus ! fichier distance au talus
    
  real                  :: bmelt_anomval        !< pour l'anomalie de bmelt appliquee
  integer               :: bmelt_anomreg        !< sur quelle region on applique l'anomalie
  real,dimension(nx,ny) :: reg_larmip           !< regions larmip
  real,dimension(nx,ny) ::  bmgrz_0               !< fusion basale a la grounding zone
  real,dimension(nx,ny) ::  bmshelf_0             !< fusion basale sous shelf

    namelist/bmelt_ant_reg_initmip/bmelt_regions,bmgrz_regions,  &  
         bmelt_talus,bmgrz_talus,bmelt_coef,file_number_shelves, &
         flag_dist,file_dist_talus,bmelt_dist0  
    namelist/bmelt_anom_larMIP/file_reg_larmip, bmelt_anomval, bmelt_anomreg 


    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
    read(num_param,bmelt_ant_reg_initmip)

    ! formats pour les ecritures dans 42
428 format(A)

    write(num_rep_42,428)'!___________________________________________________________' 
    write(num_rep_42,428) '&bmelt-ant-regions-initmip                                 '
    write(num_rep_42,*)
    write(num_rep_42,'(A,18(f0.2,","))') 'bmelt_regions      = ', bmelt_regions(:)
    write(num_rep_42,'(A,18(f0.2,","))') 'bmgrz_regions      = ', bmgrz_regions(:)
    write(num_rep_42,*) 'bmelt_talus     = ', bmelt_talus 
    write(num_rep_42,*) 'bmgrz_talus     = ', bmgrz_talus
    write(num_rep_42,*) 'bmelt_coef      = ', bmelt_coef
    write(num_rep_42,*) 'file_numer_shelves = ', file_number_shelves
    write(num_rep_42,*) 'flag_dist = ', flag_dist
    write(num_rep_42,*) 'file_dist_talus = ', file_dist_talus
    write(num_rep_42,*) 'bmelt_dist0 = ', bmelt_dist0
    write(num_rep_42,*)'/'                            
    write(num_rep_42,428) '! bmelt_regions    :  fonte basale sous shelves pour les 18 regions initMIP'
    write(num_rep_42,428) '! bmgrz_regions    :  fonte basale grounding zone pour les 18 regions initMIP'
    write(num_rep_42,428) '! bmelt_talus & bmgrz_talus  :  fonte basale apres talus cont'
    write(num_rep_42,428) '! bmelt_coef                 :  coef fonte (1 pour conserver val)'
    write(num_rep_42,428) '! file_numer_ice-shelves     : fichier zones ice shelves'
    write(num_rep_42,428) '! flag_dist : flag pour bmelt fnct distance talus'
    write(num_rep_42,428) '! file_dist_talus : fichier de distance talus'
    write(num_rep_42,428) '! bmelt_dist0 : coef de bmelt au talus'
    write(num_rep_42,*)


    ! lecture du fichier contenant les types de shelves
    ! 18 regions sur la base des bassins fournis par le projet ISMIP6-initmip antarctica
    typeshelf(:,:)=100
    file_number_shelves=TRIM(DIRNAMEINP)//trim(file_number_shelves)
    call Read_Ncdf_var('z',file_number_shelves,tab)
    typeshelf(:,:)  = tab(:,:)


    if (flag_dist.eq.1) then
       file_dist_talus=TRIM(DIRNAMEINP)//trim(file_dist_talus)
       call Read_Ncdf_var('z',file_dist_talus,tab)
       dist_talus(:,:)  = min(max(tab(:,:),0.),400.)
       coef_bmelt_dist(:,:)= 1. / ((1/bmelt_dist0)*(1+(bmelt_dist0-1)*(dist_talus(:,:)/400.)**2))
    else
       coef_bmelt_dist(:,:)= 1.
    endif

    bms_init:    do j=1,ny
       do i=1,nx

          talus:    if (Bsoc(i,j).GT.depth_talus) then

             bmshelf(i,j) = bmelt_regions(typeshelf(i,j))
             bmgrz(i,j)   = bmgrz_regions(typeshelf(i,j))

          else   ! au dela du talus continental
             
             bmshelf(i,j)=bmelt_talus  
             bmgrz(i,j)=bmgrz_talus   

          endif talus   ! fin du test sur dist_talus

       enddo
    enddo bms_init
    
    bmshelf(:,:)=bmshelf(:,:)*bmelt_coef
    bmgrz(:,:)=bmgrz(:,:)*bmelt_coef

    bmshelf_0(:,:)=bmshelf(:,:)
    bmgrz_0(:,:)=bmgrz(:,:)


  ! ______ Anomalies...
  
  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,bmelt_anom_larMIP)

  write(num_rep_42,428)'!_______________________________________________________________________' 
  write(num_rep_42,428)'!  module bmelt_ant_regions_larmip_mod                                 '
  write(num_rep_42,bmelt_anom_larMIP)
  write(num_rep_42,428)'! file_reg_larmip   = fichier regions larmip                            '
  write(num_rep_42,428)'! bmelt_anomval     = 0     ! 0=fixe, 4, 8, 16, 32                      '
  write(num_rep_42,428)'! bmelt_anomreg     = 0     ! sur quelle region (0 to 4, 5:all)       '
  write(num_rep_42,428)'!_______________________________________________________________________' 
 
    file_reg_larmip  = trim(dirnameinp)//trim(file_reg_larmip)
    call Read_Ncdf_var('regions',file_reg_larmip,tab)
    reg_larmip (:,:) = Tab(:,:)
  
    if (bmelt_anomreg.lt.6) then
       where(reg_larmip(:,:).eq.bmelt_anomreg)
          bmshelf(:,:) = bmshelf_0(:,:)  + bmelt_anomval 
          bmgrz(:,:)   = bmgrz_0(:,:)    + bmelt_anomval
       elsewhere
          bmshelf(:,:) = bmshelf_0(:,:)
          bmgrz(:,:)   = bmgrz_0(:,:)
       endwhere
    else
          bmshelf(:,:) = bmshelf_0(:,:)  + bmelt_anomval 
          bmgrz(:,:)   = bmgrz_0(:,:)    + bmelt_anomval
    endif

    
    debug_3D(:,:,33)=typeshelf(:,:)
    debug_3D(:,:,34)=bmshelf(:,:)

    
    return
  end subroutine init_bmelt

  !________________________________________________________________________________
  subroutine bmeltshelf


    ! cette routine calcule la fusion basale proprement dite

    integer :: ngr           ! nombre de voisins flottants
    real :: coef_talus       ! pour ne pas changer la fusion au dessus de l'ocean profond

    
    do j=2,ny-1
       do i=2,nx-1

          talus_nochange : if (Bsoc(i,j).GT.depth_talus) then 
             coef_talus = coefbmshelf
          else
             coef_talus = 1         ! pas de changement au dela du talus continental               
          endif talus_nochange


          shelf:    if (flot(i,j)) then    ! partie flottante

             bmelt(i,j)=coef_talus*bmshelf(i,j)*coef_bmelt_dist(i,j)

             if (fbm(i,j)) then 
                bmelt(i,j)=coef_talus*bmgrz(i,j)*coef_bmelt_dist(i,j)
             endif

             ! ATTENTION le bloc suivant sert a determiner la fusion basale d'equilibre
             ! pour les shelves stationnaires

             if (igrdline.eq.1) then
                corrbmelt(i,j)=hdot(i,j)+bmelt(i,j)   ! le bmelt d'equilibre
                debug_3D(i,j,28)=corrbmelt(i,j)
             endif


          else                   ! point pos, on compte le nombre de voisins flottants
             ngr=0
             if (flot(i+1,j)) ngr=ngr+1
             if (flot(i-1,j)) ngr=ngr+1
             if (flot(i,j+1)) ngr=ngr+1
             if (flot(i,j-1)) ngr=ngr+1

             !   la fusion des points limites est une combinaison entre valeur pose et valeur flottante
             !   en fonction du nombre de points flottants

             bmelt(i,j)= (ngr/4.*bmgrz(i,j)*coef_talus)*coef_bmelt_dist(i,j)+(1.-ngr/4.)*bmelt(i,j)


          endif shelf

       end do
    end do


    ! les bords de la grille sont mis a dist_talus=0 de force
    ! attention, pourrait poser des problemes avec AGRIF

    bmelt(1,:)=bmelt_talus   !a gauche
    bmelt(2,:)=bmelt_talus

    bmelt(nx-1,:)=bmelt_talus !  a droite 
    bmelt(nx,:)=bmelt_talus  

    bmelt(:,1)=bmelt_talus    ! en bas   
    bmelt(:,2)=bmelt_talus

    bmelt(:,ny-1)=bmelt_talus    ! en haut
    bmelt(:,ny)=bmelt_talus


    if (igrdline.eq.1) then
       bmelt(:,:)=0.      ! hdot donne alors le -bmelt
    endif

    return
  end subroutine  bmeltshelf


END MODULE  BMELT_ANT_REGIONS_LARMIP
