!> \file bmelt-seuil-profondeur_initmip_inv_mod.f90
!! Prametrise la fusion basale (ice shelves)
!! Permet de calculer par inversion la fusion basale sur chaque secteur initmip
!<

!> \namespace bmelt_seuil_prof
!! Prametrise la fusion basale (ice shelves)
!! \author C. Dumas
!! \date 02/10/2018
!! @note Pour l'actuel : 2 valeurs  pour 2 domaines de profondeur
!! + une valeur pour bmgrz
!! A choisir dans le module_choix
!! @note Used module
!! @note   - use module3D_phy
!<

module  bmelt_seuil_prof_initmip_inv         

  ! prametrise la fusion basale (ice shelves)
  ! Pour l'actuel : 2 valeurs  pour 2 domaines de profondeur
  ! + une valeur pour bmgrz
  ! A choisir dans le module_choix

  use module3d_phy
  use netcdf
  use io_netcdf_grisli

  implicit none

  real :: bm_grz                        !< valeur prescrite a la grounding zone
  real,dimension(nx,ny) ::  bmgrz       !< tabelau fusion basale a la grounding zone

  real :: bmshelf_plateau               !< valeur prescrite sur le plateau cont.
  real :: bmshelf_abysses               !< valeur prescrite au dessus de l'ocean profond
  real :: depth_talus                   !< profondeur de transition 
  real,dimension(nx,ny) ::  bmshelf     !< tableau fusion basale sous shelf
  
  
! Pour le calcul du bmelt moyen par ice-shelf
  integer,dimension(nx,ny) :: typeshelf !< Type de shelf Ronne->1 Ross ->2 ....
  integer, parameter :: nb_reg = 18
  real,dimension (nb_reg) :: bmelt_regions, bmgrz_regions
  real :: bmelt_talus,bmgrz_talus       !< Au dela du talus continental
  real :: bmelt_coef                    !< Coef pour corriger les valeurs std de bmelt (lu dans namelist mais pas utilis ici)
  integer,dimension(nb_reg):: i_bmelt   !< nbr de passage dans la boucle pour calcul de la moyenne du bmelt
  integer,dimension(nb_reg):: i_bmgrz   !< nbr de passage dans la boucle pour calcul de la moyenne du bmgrz
  
  integer :: flag_dist                  !< Pas utilise ici mais pour compatibilite lecture namelist
  

contains
  !-------------------------------------------------------------------------------
  !> SUBROUTINE: init_bmelt
  !! Cette routine fait l'initialisation pour la fusion basale.
  !! @note Elle est appele par inputfile-vec-0.5.f90
  !!
  !>
  subroutine init_bmelt



    ! Cette routine fait l'initialisation pour la fusion basale.
    ! Elle est appele par inputfile-vec-0.5.f90
    
    ! Pour le calcul du bmelt moyen par ice-shelf
    real*8, dimension(:,:),   pointer    :: tab               !< tableau 2d real pointer
    character(len=100) :: file_number_shelves ! fichier avec les zones ice-shelves
    character(len=100) :: file_bmelt_anom
    real,dimension(nx,ny) :: dist_talus   ! distance au talus
    character(len=100) :: file_dist_talus ! pas utilise ici mais pour compatibilite lecture namelist
    real :: bmelt_dist0                   ! pas utilise ici mais pour compatibilite lecture namelist
    
    namelist/bmelt_seuil/bm_grz,bmshelf_plateau,bmshelf_abysses,depth_talus
    
    namelist/bmelt_ant_reg_initmip/bmelt_regions,bmgrz_regions,  &
      bmelt_talus,bmgrz_talus,bmelt_coef,file_number_shelves,   &
      flag_dist,file_dist_talus,bmelt_dist0

    if (itracebug.eq.1)  call tracebug('entree dans init_bmelt de bmelt_seuil_prof')

    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
    read(num_param,bmelt_seuil)
    write(num_rep_42,bmelt_seuil)

    !    ecriture dans le fichier parametres

428 format(A)

    write(num_rep_42,428)'!___________________________________________________________' 
    write(num_rep_42,428) '&bmelt_seuil                     ! module  bmelt_seuil_prof'
    write(num_rep_42,*)   'bm_grz           =',bm_grz
    write(num_rep_42,*)   'bmshelf_plateau  =',bmshelf_plateau
    write(num_rep_42,*)   'bmshelf_abysses  =',bmshelf_abysses
    write(num_rep_42,*)   'depth_talus      =',depth_talus
    write(num_rep_42,*)'/'                      
    write(num_rep_42,428) '! Pour l actuel : bm_grz a la grounding line'
    write(num_rep_42,428) '!                 bmshelf_plateau sur le plateau continental'
    write(num_rep_42,428) '!                 bmshelf_abysses pour les grandes profondeurs'
    write(num_rep_42,428) '!       depth_talus, negative, separation entre les 2 domaines'
    write(num_rep_42,*)
    write(num_rep_42,428)'!___________________________________________________________' 



    bmgrz(:,:) = bm_grz

    where (Bsoc0(:,:).lt.depth_talus)
        bmshelf(:,:)=bmshelf_abysses
    elsewhere
        bmshelf(:,:)=bmshelf_plateau
    end where

    debug_3D(:,:,34)=bmshelf(:,:)
    
! Si on fait l'inversion on doit connaitre les regions pour calculer le taux de fonte par region    
    if (ibmelt_inv.eq.1) then
      rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
      read(num_param,bmelt_ant_reg_initmip)
      ! formats pour les ecritures dans 42
      write(num_rep_42,428)'!___________________________________________________________'
      write(num_rep_42,428) '&bmelt-ant-regions-initmip                                 '
      write(num_rep_42,*)
      write(num_rep_42,'(A,18(f0.2,","))') 'bmelt_regions      = ', bmelt_regions(:)
      write(num_rep_42,'(A,18(f0.2,","))') 'bmgrz_regions      = ', bmgrz_regions(:)
      write(num_rep_42,*) 'bmelt_talus     = ', bmelt_talus
      write(num_rep_42,*) 'bmgrz_talus     = ', bmgrz_talus
      write(num_rep_42,*) 'bmelt_coef      = ', bmelt_coef
      write(num_rep_42,*) 'file_numer_shelves = ', file_number_shelves
      write(num_rep_42,*) 'flag_dist = ', flag_dist
      write(num_rep_42,*) 'file_dist_talus = ', file_dist_talus
      write(num_rep_42,*) 'bmelt_dist0 = ', bmelt_dist0
      write(num_rep_42,*)'/'
      write(num_rep_42,428) '! bmelt_regions    :  fonte basale sous shelves pour les 18 regions initMIP'
      write(num_rep_42,428) '! bmgrz_regions    :  fonte basale grounding zone pour les 18 regions initMIP'
      write(num_rep_42,428) '! bmelt_talus & bmgrz_talus  :  fonte basale apres talus cont'
      write(num_rep_42,428) '! bmelt_coef                 :  coef fonte (1 pour conserver val)'
      write(num_rep_42,428) '! file_numer_ice-shelves     : fichier zones ice shelves'
!~     write(num_rep_42,428) '! flag_dist : flag pour bmelt fnct distance talus'
!~     write(num_rep_42,428) '! file_dist_talus : fichier de distance talus'
!~     write(num_rep_42,428) '! bmelt_dist0 : coef de bmelt au talus'
      write(num_rep_42,*)
      ! lecture du fichier contenant les types de shelves
      ! 18 regions sur la base des bassins fournis par le projet ISMIP6-initmip antarctica
      typeshelf(:,:)=100
      file_number_shelves=TRIM(DIRNAMEINP)//trim(file_number_shelves)
      call Read_Ncdf_var('z',file_number_shelves,tab)
      typeshelf(:,:)  = tab(:,:)
    endif
    bmelt_regions(:)=0.
    bmgrz_regions(:)=0.
    i_bmelt(:)=0
    i_bmgrz(:)=0

    return
  end subroutine init_bmelt

  !________________________________________________________________________________

  !> SUBROUTINE: bmeltshelf
  !! Cette routine calcule la fusion basale proprement dite pour les points flottants
  !! @note coefbmshelf a ete calcule par forclim et permet de faire varier en fonction du climat
  !>

  subroutine bmeltshelf


    ! cette routine calcule la fusion basale proprement dite pour les points flottants
    ! coefbmshelf a ete calcule par forclim et permet de faire varier en fonction du climat

    integer :: ngr           ! nombre de voisins flottants
    real    :: coef_talus    ! pour ne pas changer la fusion au dessus de l'ocean profond
    
    ! Pour le calcul du bmelt moyen par ice-shelf
    integer,dimension(nb_reg) :: nbr_pts_shelf, nbr_pts_grz ! nombre de points shelf et grz par zone
    real,dimension(nb_reg) :: bmelt_regions_time, bmgrz_regions_time ! Pour le caclul du bemlt/bmgrz par zone
    integer :: nregion ! pour calcul du bmelt sur toutes les regions

    if (itracebug.eq.1)  call tracebug('entree dans bmeltshelf de bmelt_seuil_prof')
    
    nbr_pts_shelf(:)=0
    nbr_pts_grz(:)=0
    bmelt_regions_time(:)=0.
    bmgrz_regions_time(:)=0.
    
    where (Bsoc0(:,:).lt.depth_talus)
        bmshelf(:,:)=bmshelf_abysses
    elsewhere
        bmshelf(:,:)=bmshelf_plateau
    end where

    do j=1,ny
      do i=1,nx

        talus_nochange: if (Bsoc0(i,j).lt.depth_talus) then 
            coef_talus = 1         ! pas de changement au dessus de l'ocecan profond
        else
            coef_talus = coefbmshelf
        endif talus_nochange


        shelf: if (flot(i,j)) then    ! partie flottante

            bmelt(i,j)=coef_talus*bmshelf(i,j)

            ! fbm est vrai si le point est flottant mais un des voisins est pose
            ! calcule dans flottab

            if (fbm(i,j)) then 
              bmelt(i,j)=coef_talus*bmgrz(i,j)
            endif

            ! ATTENTION le bloc suivant sert a determiner la fusion basale d'equilibre
            ! pour les shelves stationnaires

            if ((igrdline.eq.1).and.(ibmelt_inv.eq.1)) then
                !     corrbmelt(i,j)=hdot(i,j)+bmelt(i,j)   ! le bmelt d'equilibre
                !     bmelt(i,j)=corrbmelt(i,j)
                corrbmelt(i,j)=corrbmelt(i,j)+hdot(i,j)*0.85
                bmelt(i,j)=bmelt(i,j)+corrbmelt(i,j)
                if (H(i,j).gt.50.) then ! on ne prend en compte que les points avec H > 50m
                  if (fbm(i,j)) then ! points grounding line
                    bmgrz_regions_time(typeshelf(i,j)) = bmgrz_regions_time(typeshelf(i,j)) + bmelt(i,j)
                    nbr_pts_grz(typeshelf(i,j)) = nbr_pts_grz(typeshelf(i,j)) + 1
                  else ! points shelf
                    bmelt_regions_time(typeshelf(i,j)) = bmelt_regions_time(typeshelf(i,j)) + bmelt(i,j)
                    nbr_pts_shelf(typeshelf(i,j)) = nbr_pts_shelf(typeshelf(i,j)) + 1
                  endif
                endif
            endif

        else                   ! point pos, on compte le nombre de voisins flottants
            ngr=0
            if ((i.ne.1).and.(i.ne.nx).and.(j.ne.1).and.(j.ne.ny)) then 
                if (flot(i+1,j)) ngr=ngr+1
                if (flot(i-1,j)) ngr=ngr+1
                if (flot(i,j+1)) ngr=ngr+1
                if (flot(i,j-1)) ngr=ngr+1
            end if

            !   la fusion des points limites est une combinaison entre valeur pose et valeur flottante
            !   en fonction du nombre de points flottants

            bmelt(i,j)= ngr/4.*bmgrz(i,j)*coef_talus+(1.-ngr/4.)*bmelt(i,j)

            if ((igrdline.eq.1).and.(ibmelt_inv.eq.1)) then
                corrbmelt(i,j)=corrbmelt(i,j)+hdot(i,j)*0.85
                bmelt(i,j)=bmelt(i,j)+corrbmelt(i,j)
            endif
        endif shelf
      end do
    end do

    if ((ibmelt_inv.eq.1).and.(time.gt.10.)) then ! on ne prend pas en compte les 30 premiers pas de temps
      do nregion=1,nb_reg
        if (nbr_pts_shelf(nregion).gt.1) then
          bmelt_regions(nregion) = bmelt_regions_time(nregion)/nbr_pts_shelf(nregion) + bmelt_regions(nregion)
          i_bmelt(nregion) = i_bmelt(nregion)+1
        endif
        if (nbr_pts_grz(nregion).ge.1) then
          bmgrz_regions(nregion) = bmgrz_regions_time(nregion)/nbr_pts_grz(nregion) + bmgrz_regions(nregion)
          i_bmgrz(nregion) = i_bmgrz(nregion)+1
        endif
      enddo
      print*,'time=',time
      print*,'bmgrz des ',nb_reg,' regions'
      print*,'Ross 15 16'
      print*,bmgrz_regions_time(15)/nbr_pts_grz(15),nbr_pts_grz(15),bmgrz_regions_time(16)/nbr_pts_grz(16),nbr_pts_grz(16)
      print*,'Ronny 14 18'
      print*,bmgrz_regions_time(14)/nbr_pts_grz(14),nbr_pts_grz(14),bmgrz_regions_time(18)/nbr_pts_grz(18),nbr_pts_grz(18)
      print*,'PIG 17 et Amery 5'
      print*,bmgrz_regions_time(17)/nbr_pts_grz(17),nbr_pts_grz(17),bmgrz_regions_time(5)/nbr_pts_grz(5),nbr_pts_grz(5)
      
      if (time.eq.tend) then ! a le fin du run : calcul du bmelt et bmgrz moyen par secteur
        do nregion=1,nb_reg
          if (i_bmelt(nregion).gt.0) then
            bmelt_regions(nregion)=bmelt_regions(nregion)/i_bmelt(nregion)
          else
            bmelt_regions(nregion)=0.
          endif
          if (i_bmgrz(nregion).gt.0) then
            bmgrz_regions(nregion)=bmgrz_regions(nregion)/i_bmgrz(nregion)
          else
            bmgrz_regions(nregion)=0.
          endif
        enddo
        print*,'Fin de la simule, time=',time
        print*,'calcul des bmelt et bmgrz par inversion :'
        print*,'bmelt des ',nb_reg,' regions'
        print*,bmelt_regions(:)
        print*,'bmgrz des ',nb_reg,' regions'
        print*,bmgrz_regions(:)
      endif
    endif
          
    if ((igrdline.eq.1).and.(ibmelt_inv.eq.0)) then
      where (i_Hp(:,:).eq.1)
        bmelt(:,:)=0.    ! hdot donne alors le -bmelt
      endwhere
    endif
    
return
end subroutine  bmeltshelf



end module  bmelt_seuil_prof_initmip_inv
