!> \file dragging_prescr_beta_perturb_mod.f90
!! Module pour calculer le beta a partir d'une carte sur les noeuds centres.
!! perturbation du beta par paliers
!<


module dragging_prescr_beta_perturb


  use module3d_phy
!  use interface_input
  use fake_beta_iter_vitbil_mod  
  use netcdf
  use io_netcdf_grisli
  
  implicit none
  real, dimension(nx,ny) :: Vcol_x           !< uniquement pour compatibilite
  real, dimension(nx,ny) :: Vcol_y           !<
  real, dimension(nx,ny) :: Vsl_x            !<
  real, dimension(nx,ny) :: Vsl_y            !<


  real, dimension(nx,ny) :: drag_centre      !< beta on major node (average)
  real, dimension(nx,ny) :: betamoy_x        !< pour faire les moyennes glissantes
  real, dimension(nx,ny) :: betamoy_y        !<
  integer, dimension(nx,ny) :: nbvoisx       !<
  integer, dimension(nx,ny) :: nbvoisy       !<


  real :: beta_limgz                         !< when beta gt beta_limgz -> not gzmx
  real :: beta_min                           !< minimum value of beta
  real :: beta_mult                          !< coefficient of beta field
  integer  :: ill,jll,nmoy

  real :: maxi                               !< calcul correction dS a la grounding line
  real :: mini

  logical :: corr_def = .False.               !< for deformation correction (compatibility)
  
  real :: begin_pert                          !< time when perturbation begin
  real :: end_pert                            !< time when perturbation end
  real :: coef_pert                           !< perturbation coeficient
  real :: coef_pert_time                      !< coef perturbation at time
  real, dimension (nx,ny) :: basinNumber      !< IMBIE2 oc. basin identifier, input
  integer :: basin_pert                       !< basin number perturbated

contains

  !-----------------------------------------------------------------------------------------
  !> SUBROUTINE: init_dragging 
  !! Cette routine fait l'initialisation du dragging.
  !<
  subroutine init_dragging      ! Cette routine fait l'initialisation du dragging.
                                ! nouvelle version : lit les fichiers nc
    implicit none
    character(len=100) :: beta_c_file               !  beta on centered grid
    character(len=100) :: file_basinNumbers         !  file read
    character(len=100) :: file_inputs               !< files read
    double precision, dimension(:,:), pointer :: tab2d => null()   !< 2d array real pointer, needed for netcdf readings
    
    namelist/beta_prescr/beta_c_file,beta_limgz,beta_min,beta_mult
    
    namelist/beta_prescr_perturb/begin_pert,end_pert,coef_pert,file_basinNumbers,basin_pert

    if (itracebug.eq.1)  call tracebug(' Prescr_beta       subroutine init_dragging')

    inv_beta=0

    ! lecture des parametres du run                    
    !--------------------------------------------------------------------

    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
428 format(A)
    read(num_param,beta_prescr)

    write(num_rep_42,428) '!___________________________________________________________' 
    write(num_rep_42,428) '!  read beta on centered grid'
    write(num_rep_42,beta_prescr)                    
    write(num_rep_42,428) '!  beta_file : nom des fichier qui contiennent les beta_c'
    write(num_rep_42,428) '!  above beta_limgz, gzmx is false'
    write(num_rep_42,428) '!  if grounded, beta_min minimum value '
    write(num_rep_42,428) '!  beta_mult : coefficient just after reading' 
    write(num_rep_42,*)
    write(num_rep_42,428) '!___________________________________________________________' 
   
    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
    read(num_param,beta_prescr_perturb)
    write(num_rep_42,428) '!___________________________________________________________'
    write(num_rep_42,428) '!  module dragging_prescr_beta_perturb'
    write(num_rep_42,beta_prescr_perturb)
    write(num_rep_42,428) '!  begin_pert : debut de la perturbation du beta'
    write(num_rep_42,428) '!  end_pert   : fin de la perturbation du beta'
    write(num_rep_42,428) '!  coef_pert  : coeficient de la perturbation'
    write(num_rep_42,428) '!  file_basinNumbers : identifier for the Imbie2 basins'
    write(num_rep_42,428) '!  basin_pert : basin number perturbated'
    write(num_rep_42,428) '!___________________________________________________________'


    beta_c_file = trim(dirnameinp)//trim(beta_c_file)
    betamax = beta_limgz
    betamax_2d(:,:) = betamax

!cdc    call lect_input(1,'beta_c',1,drag_centre,beta_c_file,'')
    
    call Read_Ncdf_var('z',beta_c_file,tab2d)
    drag_centre(:,:)=tab2d(:,:)
    
    call Read_Ncdf_var('basinNumber',TRIM(dirnameinp)//trim(file_basinNumbers),tab2d)
    basinNumber(:,:)  = tab2d(:,:)

! multiplication par le coefficient beta_mult

    drag_centre(:,:) = drag_centre(:,:)*beta_mult

    where (mk_init(:,:).eq.-2)                                             ! iles a probleme
       drag_centre(:,:) = 2.* abs(beta_limgz)
    end where


    where ((H(:,:).lt.0.1).and.(Bsoc(:,:).gt.0))                            ! zones actuellement non couvertes
       drag_centre(:,:) = 1000.                                             ! valeur relativement faible qui evite
    end where                                                               ! les freinages


     call  beta_c2stag(nx,ny,drag_mx,drag_my,drag_centre)         ! redistribute on staggered grid

     if  (beta_limgz.gt.0.) then
        beta_centre(:,:) =  drag_centre(:,:)
        betamx(:,:)      =  drag_mx(:,:)
        betamy(:,:)      =  drag_my(:,:)

     else if (beta_limgz.lt.0.) then                              ! attention sans doute pour plastic

        beta_centre(:,:) = - beta_limgz
        betamx(:,:)      = - beta_limgz
        betamy(:,:)      = - beta_limgz
        drag_centre(:,:) = - beta_limgz 
        drag_mx(:,:)     = - beta_limgz
        drag_my(:,:)     = - beta_limgz 
        beta_limgz = 1.
     end if


! divers traitements qui peuvent etre appliques a beta : mis sous forme de routine

    ! call beta_plateau_slope   !  give value beta_limgz to places where slope is >1.e-2
    ! call beta_stag2c          !  average the staggered values on central ones
    ! call beta_plateau_S       !  give value beta_limgz to places where S > 2500 m

   call beta_min_value(beta_min)      !  valeur mini que peut avoir beta (en Pa an m-1)
                                ! on peut envisager une valeur ~ 10 
                                ! rappel : beta doit tre positif.

    ! call  beta_c2stag(nx,ny,drag_mx,drag_my,drag_centre)         ! redistribute on staggered grid


    !  mstream attribution
    !________________________
    ! mstream -> regions where streaming is allowed
    ! drag_mx -> value of beta if an ice stream is allowed
    ! betamx  -> value of dragging (may depend on other conditions)

    ! to ensure having some "Dirichlet" points, there must be at least 
    ! a mstream=0 in every island


! un tour de lissage
!    call beta_stag2c(nx,ny,drag_mx,drag_my,drag_centre)                  ! average  staggered 
!                                                                         ! on central 

!    call beta_c2stag(nx,ny,drag_mx,drag_my,drag_centre)                  ! centered distributed
!                                                                         ! on staggered
!


    beta_centre(:,:) =  drag_centre(:,:)                                 ! surtout pour sorties



    where (drag_mx(:,:).ge.beta_limgz)
       mstream_mx(:,:) = 0
       betamx(:,:)     = beta_limgz
       drag_mx(:,:)    = beta_limgz
    elsewhere
       mstream_mx(:,:) = 1
       betamx(:,:)     = drag_mx(:,:)
    endwhere

    where (drag_my(:,:).ge.beta_limgz)
       mstream_my(:,:) = 0
       betamy(:,:)     = beta_limgz
       drag_my(:,:)    = beta_limgz
    elsewhere
       mstream_my(:,:) = 1
       betamy(:,:)     = drag_my(:,:)
    endwhere

    if (itracebug.eq.1)  call tracebug(' Prescr_beta  apres lecture')
!    mstream_mx(:,:) = 0
!    mstream_my(:,:) = 0

! mask prennant en compte les secteurs :
    if (basin_pert.ge.0) then
    where ((basinNumber(:,:).eq.basin_pert).and.(mstream_mx(:,:).eq.1))
      mstream_mx(:,:)=1
    elsewhere
      mstream_mx(:,:)=0
    endwhere
    where ((basinNumber(:,:).eq.basin_pert).and.(mstream_my(:,:).eq.1))
      mstream_my(:,:)=1
    elsewhere
      mstream_my(:,:)=0
    endwhere
    endif
    


    call gzm_beta_prescr
!    call write_datfile2(nx,ny,betamx,betamy,'beta-LBq15-08.dat')

!    call make_gaussienne(n_g,sigma_beta,M_gauss)

    return

  end subroutine init_dragging
  !________________________________________________________________________________

  !> SUBROUTINE: dragging 
  !! Defini la localisation des streams et le frottement basal
  !<
  !-------------------------------------------------------------------------
  subroutine dragging   ! defini la localisation des streams et le frottement basal
  
    implicit none

    if (itracebug.eq.1)  call tracebug(' beta_prescr         subroutine dragging')

!~ !   tentative d'iteration

!~ !    call beta_stag2c(nx,ny,drag_mx,drag_my,drag_centre)                  ! average  staggered 
!~                                                                           ! on central 
!~ !   call iter_gauss(n_g,nx,ny,M_gauss,-1.,drag_centre,corrstream_centre,hdot)  ! correct central
!~ !   call beta_c2stag(nx,ny,drag_mx,drag_my,corrstream_centre)             ! centered distributed
!~                                                                           ! on staggered
!~ !    drag_centre(:,:) = corrstream_centre(:,:)
!~ !    beta_centre(:,:) =  drag_centre(:,:)                                 ! surtout pour sorties

!~     ! pour l'instant pas d'autre condition que mstream
!~     where (mstream_mx(:,:).eq.1)
!~        betamx(:,:) = drag_mx(:,:)
!~     elsewhere 
!~        betamx(:,:)= beta_limgz
!~     end where

!~     where (mstream_my(:,:).eq.1)
!~        betamy(:,:) = drag_my(:,:)
!~     elsewhere 
!~        betamy(:,:)= beta_limgz
!~     end where
    

! modification de betamx et betamy en fonction du temps :
    if (time.le.begin_pert) then  ! avant perturbation
      betamx(:,:)=drag_mx(:,:)
      betamy(:,:)=drag_my(:,:)
    else if ((time.gt.begin_pert).and.(time.le.end_pert)) then ! augmentation de la perturbation jusqu'a end_pert
      coef_pert_time = 1. + ((time-begin_pert)/(end_pert-begin_pert))*coef_pert
      where (mstream_mx(:,:).eq.1)
        betamx(:,:)=max(drag_mx(:,:)*coef_pert_time,beta_min)
      else where
        betamx(:,:)=drag_mx(:,:)
      end where
      where (mstream_my(:,:).eq.1)
        betamy(:,:)=max(drag_my(:,:)*coef_pert_time,beta_min)
      else where
        betamy(:,:)=drag_my(:,:)
      end where
    else ! perturbation max apres end_pert
      where (mstream_mx(:,:).eq.1)
        betamx(:,:)=max(drag_mx(:,:)*coef_pert_time,beta_min)
      else where
        betamx(:,:)=drag_mx(:,:)
      end where
      where (mstream_my(:,:).eq.1)
        betamy(:,:)=max(drag_my(:,:)*coef_pert_time,beta_min)
      else where
        betamy(:,:)=drag_my(:,:)
      end where
    endif
!    print*,'dragging_prescr_pert betamx=',time, betamx(149,148),coef_pert_time
    call  gzm_beta_prescr        ! determine gz et flgz et met a 0 le beta des points flottants
!    print*,'betamx',betamx(149,148)

! pour avoir les bonnes valeurs dans la sortie beta_c :
    do j=2,ny-1
      do i=2,nx-1
        beta_centre(i,j) =  (betamx(i,j)+betamx(i+1,j)+betamy(i,j)+betamy(i,j+1))/4.
      enddo
    enddo
    
    return
  end subroutine dragging

  !----------------------------------------------------------------------------------
  !>SUBROUTINE: gzm_beta_prescr
  !! Calcul de gzmx
  !<

  subroutine gzm_beta_prescr     ! attribue flgzmx et gzmx (et my)
    
    logical ::  test_Tx          ! test if there is a basal melting point in the surrounding
    logical ::  test_Ty          ! test if there is a basal melting point in the surrounding
    logical ::  test_beta_x      ! test if there is a low drag point in the surrounding
    logical ::  test_beta_y      ! test if there is a low drag point in the surrounding

    real    ::  beta_forc_fleuve ! below this value -> ice stream

!    beta_forc_fleuve = 5.e3
    beta_forc_fleuve = beta_limgz

    ! calcul de gzmx 


    gzmx(:,:)=.true.
    gzmy(:,:)=.true.
    flgzmx(:,:)=.false.
    flgzmy(:,:)=.false.


    ! points flottants : flgzmx mais pas gzmx
    do j=2,ny
       do i=2,nx
          if (flot(i,j).and.(flot(i-1,j))) then
             flotmx(i,j)=.true.
             flgzmx(i,j)=.true.
             gzmx(i,j)=.false.
             betamx(i,j)=0.
          end if
          if (flot(i,j).and.(flot(i,j-1))) then
             flotmy(i,j)=.true.
             flgzmy(i,j)=.true.
             gzmy(i,j)=.false.
             betamy(i,j)=0.
          end if
       end do
    end do

    if (itracebug.eq.1)  call tracebug(' apres criteres flot')

    if (itracebug.eq.1) write(num_tracebug,*) 'gzmx', gzmx(127,127) 

    !--------- autres criteres

    ! points poses
    gzmx(:,:)=gzmx(:,:).and.(betamx(:,:).lt.beta_limgz)   !  Pas de calcul pour les points
    gzmy(:,:)=gzmy(:,:).and.(betamy(:,:).lt.beta_limgz)   !  au fort beta



    ! critere (lache) sur la temperature
    do j = 2, ny-1
       do i =2, nx-1 

!   test s'il y a un point tempere dans les environs
          test_Tx = any( ibase( i-1:i   , j-1:j+1 ).gt.1)
          test_Ty = any( ibase( i-1:i+1 , j-1:j   ).gt.1)

!   test s'il y a un point low drag dans les environs
          test_beta_x = any( drag_centre( i-1:i   , j-1:j+1 ) .lt. beta_forc_fleuve )
          test_beta_y = any( drag_centre( i-1:i+1 , j-1:j   ) .lt. beta_forc_fleuve )

!   critere pour gzmx

! Attention : les deux lignes suivants sont en commentaire pour voir l'effet

!          gzmx(i,j) =  gzmx(i,j) .and. (test_Tx.or.test_beta_x)
!          gzmy(i,j) =  gzmy(i,j) .and. (test_Ty.or.test_beta_y)

       end do
    end do


    if (itracebug.eq.1)  call tracebug(' apres autres criteres ')
    if (itracebug.eq.1) write(num_tracebug,*) 'gzmx', gzmx(127,127) 

    flgzmx(:,:) = flgzmx(:,:) .or. gzmx(:,:)
    flgzmy(:,:) = flgzmy(:,:) .or. gzmy(:,:)

    where ((.not.flotmx(:,:)).and.(.not.gzmx(:,:)))
       betamx(:,:) = beta_limgz
    end where

    where ((.not.flotmy(:,:)).and.(.not.gzmy(:,:)))
       betamy(:,:) = beta_limgz
    end where

	fleuvemx(:,:)=gzmx(:,:)
	fleuvemy(:,:)=gzmy(:,:)
    
  end subroutine gzm_beta_prescr


  !----------------------------------------------------------------------------------
  ! Routines qui permettent des manip simples sur beta.


!______________________________________________________________________________________
!>SUBROUTINE:  beta_plateau_slope 
!! give value beta_limgz to places where slope is >1.e-2 
!<

  subroutine beta_plateau_slope      ! give value beta_limgz to places where slope is >1.e-2

    call slope_surf

    where (abs(sdx(:,:)).gt.1.e-2)
       drag_mx(:,:) = beta_limgz
    end where

    where (abs(sdy(:,:)).gt.1.e-2)
       drag_my(:,:) = beta_limgz
    end where
  end subroutine beta_plateau_slope
!______________________________________________________________________________________
!> SUBROUTINE beta_plateau_S 
!! give value beta_limgz to places where S > 2500 m
!<
  subroutine beta_plateau_S     !  give value beta_limgz to places where S > 2500 m

    where (S(:,:).gt.2500.)     ! moyen simple de nettoyer l'interieur. WAIS pas touche
       drag_centre(:,:)=beta_limgz
       drag_mx(:,:)=beta_limgz
       drag_my(:,:)=beta_limgz
    end where
  end subroutine beta_plateau_S
!______________________________________________________________________________________
!>SUBROUTINE:   beta_min_value 
!! valeur mini que peut avoir beta (en Pa an m-1) 
!<

  subroutine beta_min_value(val_betamin)

  real :: val_betamin     ! valeur mini que peut avoir beta (en Pa an m-1)

    drag_centre(:,:) = max(drag_centre(:,:),val_betamin)
    drag_mx(:,:) = max(drag_mx(:,:),val_betamin)
    drag_my(:,:) = max(drag_my(:,:),val_betamin)
    where(flot(:,:))
       drag_centre(:,:) = 0.
    end where

  end subroutine beta_min_value
!______________________________________________________________________________________
!>SUBROUTINE:   beta_stag2c 
!! average the staggered values on central ones
!<

  subroutine beta_stag2c(nxx,nyy,R_mx,R_my,R_centre)      ! average the staggered values on central ones

    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(in)   ::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(in)   ::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(out)  ::  R_centre               ! valeur du beta sur maille centree   
    
    R_centre(:,:)=0.
    do j=2,ny-1
       do i=2,nx-1
          R_centre(i,j)= ((R_mx(i,j)+R_mx(i+1,j)) &
               + (R_my(i,j)+R_my(i,j+1)))*0.25
       end do
    end do
  end subroutine beta_stag2c
!______________________________________________________________________________________
!>SUBROUTINE:   beta_c2stag  
!! redistribute central values on staggered grid
!<

  subroutine beta_c2stag(nxx,nyy,R_mx,R_my,R_centre)              ! redistribute central values on staggered grid

    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(out)  ::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(out)  ::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(in)   ::  R_centre               ! valeur du beta sur maille centree

    do j=2,nyy 
       do i=2,nxx
          R_mx(i,j)= (R_centre(i-1,j)+R_centre(i,j))*0.5
          R_my(i,j)= (R_centre(i,j-1)+R_centre(i,j))*0.5
       end do
    end do

  end subroutine beta_c2stag
!______________________________________________________________________________________
!>SUBROUTINE:   beta_c2stag_min  
!! redistribute central values on staggered grid
!<

  subroutine beta_c2stag_min(nxx,nyy,R_mx,R_my,R_centre)              ! redistribute central values on staggered grid

    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(inout)::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(inout)::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(in)   ::  R_centre               ! valeur du beta sur maille centree

    do j=2,nyy 
       do i=2,nxx
          R_mx(i,j)= min(R_mx(i,j),(R_centre(i-1,j)+R_centre(i,j))*0.5)
          R_my(i,j)= min(R_my(i,j),(R_centre(i,j-1)+R_centre(i,j))*0.5)
       end do
    end do

  end subroutine beta_c2stag_min
!______________________________________________________________________________________
!>SUBROUTINE:   make_gaussienne 
!! cree une matrice gaussienne normalisee a 1
!<

  subroutine  make_gaussienne(n,sigma,M)

    implicit none
    integer,intent(in)                       :: n          ! demi taille du tableau gaussienne
    real, intent(in)                         :: sigma      ! sigma gaussienne exprime en dx
    real,dimension(-n:n,-n:n),intent(inout)  :: M          ! tableau gaussienne


!   variables locales
    real                                     :: dist2      ! distance au centre (au carre)
    real                                     :: som        ! pour calibration
    real                                     :: sigma2     ! pour gaussienne

    som = 0.
    sigma2 = sigma * sigma * 2
    do j = -n, n
       do i = -n, n
          dist2 = (i*i+j*j)
          M(i,j) = exp(-dist2/sigma2)
          som=som+M(i,j)
       end do
    end do

    M(:,:) = M(:,:) / som

!    do j = -n, n
!       write(6,'(5(f0.4,1x))') (M(i,j),i=-n,n)
!    end do

  end subroutine make_gaussienne

!______________________________________________________________________________________
!>SUBROUTINE:  iter_gauss
!! utilise hdot pour faire les iterations sur beta_centre
!<
  subroutine iter_gauss(n,nxx,nyy,M,coef,beta_in,beta_out,Ecart)

    integer,                  intent(in)    :: n          ! taille matrice gaussienne 
    integer,                  intent(in)    :: nxx,nyy    ! taille du domaine
    real,dimension(-n:n,-n:n),intent(in)    :: M          ! matrice gaussienne 
    real,                     intent(in)    :: coef       ! coefficient multiplicateur
    real,dimension(nx,ny),    intent(in)    :: beta_in    ! la valeur initiale
    real,dimension(nx,ny),    intent(out)   :: beta_out   ! la valeur apres iteration
    real,dimension(nx,ny),    intent(in)    :: Ecart      ! l'ecart a l'obs

! variables locales
    integer                                 :: ii,jj

    do j= 1 + n, nyy - n
       do i = 1 + n, nxx - n
          beta_out(i,j) = beta_in(i,j)

          do jj = -n, n
             do ii = -n, n 
                beta_out(i,j) = beta_out(i,j) + coef * M(ii,jj) * ecart(i+ii,j+jj)
             end do
          end do

       end do
    end do

    beta_out(:,:) = max (beta_out(:,:),0.)
  end subroutine iter_gauss

end module dragging_prescr_beta_perturb

!!$! add the nodes of topographical instability (on major nodes)
! a garder eventuellement pour runs instabilite
!!$beta_file = trim(dirnameinp)//'sensib_H_Gael-15km.dat'
!!$call lect_datfile(nx,ny,drag_instab_topo,1,beta_file)   
!!$
!!$where (drag_instab_topo(:,:).gt.0.)
!!$   drag_centre(:,:)=min(drag_centre(:,:),400.)
!!$endwhere
