!> \file steps_time_loop.f90
!! TOOOOOOOOO DOOOOOOOOOOOOO
!! Step_grisli contient
!<

!--------------------------------------------------------------------------
! step_time_loop est a l'interieur de la boucle temps. 
! il commence par icethick, qui dtermine dt, isynchro

! ensuite contient les appels aux diverses sorties
! puis un appel a step_grisli


subroutine step_time_loop        

  use module3d_phy
  use module_choix ! module de choix du type de run
                   !  module_choix donne acces a tous les modules
                   !  de declaration des packages
  use sorties_ncdf_grisli
  use flottab_mod
  use icetempmod
  use diagno_mod
  use bilan_eau_mod
  use bilan_flux_mod
  !  use track_debug 

  implicit none
  integer :: nt_init = 0      ! number of loops for initialisation of ice thickness

  if (itracebug.eq.1)  call tracebug('Entree dans step_time_loop ')
  if (itracebug.eq.1) write(num_tracebug,*) 'nt = ',nt


  ! ice thickness evolution
  !=============================


  spinup_icethick: if (ispinup.eq.0.or.ispinup.eq.2.or.nt.lt.nt_init) then 

     !-------------------------------------------------------------------
     ! spinup_icethick : Le calcul changement d'epaisseur (icethick)
     !----------------
     !   - est fait dans tous les cas pour les premieres boucles temporelles
     !   - pour ispinup = 0 (run standard)
     !   - pour ispinup = 2 (force avec vitesses de bilan)
     !
     ! else 
     !   juste un increment pour le temps 
     !    -  pour ispinup = 1 et  ispinup = 3  temperature calculation
     !  
     !-------------------------------------------------------------------

     if (itracebug.eq.1)  call tracebug(' Dans spinup_icethick : appelle icethick')

     call icethick3
     call flottab
     call determin_tache
     call calving
     call ablation_bord
     call bilan_eau
     call bilan_flux_output     
     call flottab
     if (isynchro.eq.1.or.nt.eq.1) call determin_marais
     if (itracebug.eq.1)  call tracebug('apres marais')

     ! new calculation of ice surface elevation
     where(.not.flot(:,:))
        S(:,:)=Bsoc(:,:)+H(:,:)
        S(:,:)=max(S(:,:),sealevel_2d(:,:)) !afq -- WARNING: est-ce qu'on veut vraiment mettre S a la valeur locale du niveau marin?
        B(:,:)=Bsoc(:,:)
     end where

     ! calcul de Hmx et Hmy -> shift=-1, dim=1 -> H(i-1,j)

     hmx(:,:)=0.5*(H(:,:)+eoshift(H(:,:),shift=-1,boundary=0d0,dim=1))
     hmy(:,:)=0.5*(H(:,:)+eoshift(H(:,:),shift=-1,boundary=0d0,dim=2))
     hmx(1,:)=0.
     hmy(:,1)=0.

     ! mise a 0 des vitesses quand epaisseur nulle
     where (hmx(:,:).le.1.)
        uxbar(:,:)=0.
     endwhere
     where (hmy(:,:).le.1.)
        uybar(:,:)=0.
     endwhere

  else   ! time increment when no icethick : ispinup= 1 or 3    ! loop spinup_icethick

     hdot(:,:)=0.
     time=nt*dtmax

     if (itracebug.eq.1) write(num_tracebug,*) ' Dans spinup_icethick:  no icethick ',ispinup, &
          '    time=',time


  end if spinup_icethick
  ! end of ice thickness evolution
  !====================================================================



  ! outputs
  !====================================================================

  !   vertical plan snapshots (profiles)
  !------------------------------------------------------------------

  !if ((NT.eq.1) &    !   .or.(NT.eq.2)
  !     .or.(mod(abs(dble(TIME)),dble(DTPROFILE)).lt.dble(dtmin)) &
  !     .or.(abs(TIME-TEND).lt.dtmin)) then
  !  call sortieprofile()
  !endif

  !  sorties netcdf Hassine (aout 2010)  (2D and 3D)
  !------------------------------------------------------------------

  call testsort_time_ncdf(time)
  if (iglob_ncdf .EQ. 1) call sortie_ncdf_cat
!  if ((isynchro.eq.1).or.(nt.eq.1).and.(mod(abs(TIME),1.).lt.dtmin)) then
  if ((nt.eq.1).or.((isynchro.eq.1).and.(mod(abs(TIME),1.).lt.dtmin))) then
    call shortoutput
    diff_H = 0.
    Bm_dtt(:,:) = 0.
    bmelt_dtt(:,:) = 0.
    calv_dtt(:,:)=0.
    ablbord_dtt(:,:)=0.
    diff_H_2D(:,:)=0.
    grline_dtt(:,:)=0.
  endif

  !   sortie compteur tous les dtcpt ans 
  !------------------------------------------------------------------
  !iout == 1 sortie cptr
  !iout == 2 sortie nc
  if (itracebug.eq.1)  call tracebug('dans steps_time_loop avant call out_recovery ')

  call out_recovery(iout)
  
  ! end of outputs

  !====================================================================
  ! 
  call step_thermomeca()
  !
  ! contient tout le reste de la dynamique et du forage 
  !====================================================================

end subroutine step_time_loop






!---------------------------------------------------------------------
! The subroutine  step_thermomeca() calls the following processes
!
! - climatic forcing and its impact on surface mass balance
! - dynamic velocities (SIA)
! - ice temperature

subroutine step_thermomeca()


  use module3d_phy
  use module_choix ! module de choix du type de run
  !  module_choix donne acces a tous les modules
  !  de declaration des packages
  use icetempmod
  use sorties_ncdf_grisli
  use flottab_mod
  use diagno_mod
  use resolmeca_SIA_L1
  use iso_declar, only: dt_iso
!  use track_debug 


  implicit none

  integer :: m
  logical :: shelf_vitbil = .true.   ! si vrai on prend les vitesses de bilan pour le shelf
  integer :: nt_init_tm = 0          ! number of loops for initialisation of thermo mechanical
  integer :: iter_visco              ! number of iterations for ssa viscosity

  if (ispinup.le.1) shelf_vitbil = .false.         ! general case, ice shelves velocities are computed by diagnoshelf

  timemax=time
  if (itracebug.eq.1) write(num_tracebug,*) 'entree dans step_thermomeca', &
       '   nt=',nt

  !--------------------------------------------------------------------
  !     lecture dans un petit fichier "runname.limit" au cas ou les
  !     parametres du run seraient changes  
  !                                                   
  !--------------------------------------------------------------------

!  call limit_file(nt,real(time),dt,tend,dtsortie,dtcpt,testdiag,dtt,runname)

  !if (time.lt.tgrounded) then
  !   marine=.false.
  !else
  marine=.true.
  !endif

  ! update the regions (floating, ice ...)
  !-------------------------------------------
  call masque()


  !-----------------------------------------------------------------------!
  !   debut d'un bloc appels tous les dtt (pas de temps long- asynchrone) !
  !-------------   -------------------------------------------------------!



  isync_1: if (ISYNCHRO.eq.1) then          ! debut bloc appels tous les dtt    

     ! climatic forcing
     !=====================
     call forclim                              
     call ablation

     ! oceanic forcing
     !=====================
     ! because eustatic sea level is updated in forclim, we need to call the RSL here
     call rsl         ! provide the map of the local sea level (sealevel_2d)
     call bmeltshelf  ! afq -- icetemp is not called before the 4th dtt, so a first call to bmelt here

     ! SIA dynamic velocities  (deformation + loi de glissement )
     !========================

     ! remark : velocity is 3D
     !          to update the velocity for ice temperature calculation
     !            - surface mass balance for velocity (because ablation was called)
     !            - impact of surface slope


     spinup_1_veloc: if ((ispinup.ne.2).or.(nt.lt.nt_init_tm)) then        ! cas general

        !---------------------------------------------------------------------------------
        ! spinup_1 : calcul des vitesses dynamiques :
        !------------
        !            - est fait dans tous les cas pour les premieres boucles temporelles
        !            - est fait pour ispinup=3 pour calibration forme de vitesse 
        !            - n'est pas fait pour ispinup=2 (test balance vel sur icethick)
        !
        !---------------------------------------------------------------------------------

        if (itracebug.eq.1)  call tracebug(' Dans spinup_1_veloc : calculer SIA_velocities')

        call Neffect()                                             
        call SIA_velocities()

     else    ! ispinup = 2

        call force_balance_vel   ! fait l'ensemble du champ (flgz ou pas)

     endif spinup_1_veloc


     ! ice temperature 
     !===================================================================

     ! update values in the structures Geom_g, Temperature_g, ...

! afq     calc_temp:     if ((nt.gt.2).and.(geoplace(1:5).ne.'mism3')) then
     calc_temp:     if (geoplace(1:5).ne.'mism3') then
        if (itracebug.eq.1)  call tracebug('avant appel icetemp')
        call icetemp
        ! subroutines pour le calcul de la fusion basale
        ! afq -- 28-10-2016: there is a reset of bmelt in temp_col in the
        ! current version of the code, so shelf and grounded melt have to 
        ! be called here. TODO: remove reset in icetemp routines??
        call bmeltshelf
        call bmelt_grounded

     endif calc_temp


     ! flowlaw 
     !===================================================================

     spinup_2_flowlaw: if ((ispinup.ne.2).or.(nt.lt.nt_init_tm)) then    ! cas general

        !---------------------------------------------------------------------------------
        ! spinup_2_flowlaw
        !----------------
        ! si ispinup =   2 , on ne fait pas le calcul de flowlaw, ni de diffusiv 
        !                    puisque les vitesses de bilan sont imposees 
        !                                             (initialisee nt < nt_init_tm)
        !                   
        ! Flowlaw et diffusiv : servent soit directement (ispinup = 0,1) soit pour le 
        !                       rescaling (ispinup = 3) 
        !---------------------------------------------------------------------------------

        if (itracebug.eq.1)  call tracebug(' Dans spinup_2_flowlaw')

           call flow_general
           do iglen=n1poly,n2poly
              call flowlaw(iglen)
           end do


     end if spinup_2_flowlaw

     if (ispinup.eq.3) then
        call calc_coef_vitbil                    ! recalibre sa_mx et s2a_mx et ddbx pour toute la zone posee
     end if                                      ! Dans le shelf flgz = Ubar = Vbil et udef=0

     call tracer      ! aurel neem (attention, position a verifier)   

  endif isync_1


  ! fin du bloc tous les dtt isync_1
  !------------------------------------


  spinup_4_vitdyn: if  ((ispinup.le.1).or.(nt.lt.nt_init_tm)) then  

     !---------------------------------------------------------------------------------
     ! spinup_4_vitdyn
     !----------------
     ! Passe dans ce bloc que quand on veut calculer les vitesses dynamiques :
     ! - initialisations (nt.lt.nt_init_tm)
     ! - run standard  (ispinup=0)
     ! - spinup temperature avec vitesses dynamiques  (ispinup = 1)                
     !---------------------------------------------------------------------------------



     call Neffect()

     call diffusiv()

     ! strain rate calculations
     !===========================

     call strain_rate()

     ! ice shelves and ice streams velocities : SSA
     !==============================================

     !-----------------------------------------------------------------------
     !              debut d'un deuxieme bloc appels tous les dtt                         
     !-------------   -------------------------------------------------------

     isync_2:    if ((shelfy).and.(marine).and.(isynchro.eq.1)) then                

        !les 3 lignes ci-dessous pour avoir les champs avant diagno (en cas de pb colineaire)
        !call init_sortie_ncdf
        !call sortie_ncdf_cat

        if (icompteur.eq.1) then    ! reprise de tout le vecteur d'etat    
           iter_visco = 1

        else if ((icompteur.ne.1).and.(nt.le.1)) then  ! initialisation avec reprise partielle 
                                                       ! ou nulle du vecteur d' etat
          iter_visco = 10
        else
          iter_visco = 2
        end if
 
           test_iter_diag=0.
           do m=1,iter_visco              
              call diagnoshelf
              call mix_SIA_L1  
              call strain_rate
              if (test_iter_diag.lt.1.e-2) exit
           end do
           
           ! iterations pour inversion du beta en fonction des vitesses de bilan
           if ((inv_beta.eq.1).and.(time.ge.time_iter).and.(time.le.time_iter_end)) then  ! iterations pendant 20 ans
              do m=1,nb_iter_vitbil
                 call beta_iter_vitbil(m)
                 call diagnoshelf
                 call mix_SIA_L1
                 call strain_rate
              end do
           end if
           ! fin des iterations beta_iter_vitbil

        endif isync_2

     call  mix_SIA_L1  

  end if spinup_4_vitdyn
 
  
  ! isostasy
  !=================

  spinup_3_bed: if ((ISYNCHRO.eq.1).and.(ispinup.eq.0.or.nt.lt.nt_init_tm)) then
  !---------------------------------------------------------------------------------
  ! spinup_3_bed
  !----------------
  ! Ne passe dans ce bloc que quand on veut l'isostasie, donc si variations epaisseur
  ! run standard  (ispinup=0) et pour les initialisations                  
  !---------------------------------------------------------------------------------

  if (itracebug.eq.1)  call tracebug(' Dans spinup_3_bed')

  if ((nbed.eq.1).and.nt.ne.1.and.isynchro.eq.1.and.(mod(abs(TIME),dt_iso).lt.dtmin)) then
    call bedrock                 !  bedrock adjustment
  endif


  end if spinup_3_bed
  

end subroutine step_thermomeca
